@@ -13,10 +13,10 @@ class i386Module(envi.ArchitectureModule):
         return i386RegisterContext()
 
     def archGetBreakInstr(self):
-        return '\xcc'
+        return b'\xcc'
 
     def archGetNopInstr(self):
-        return '\x90'
+        return b'\x90'
 
     def archGetRegisterGroups(self):
         groups = envi.ArchitectureModule.archGetRegisterGroups(self)
