@@ -8,6 +8,7 @@ import pprint
 import socket
 import logging
 import traceback
+import collections
 from getopt import getopt
 
 import vtrace
@@ -38,7 +39,7 @@ from vivisect.const import *
 logger = logging.getLogger(__name__)
 
 
-class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
+class VivCli(e_cli.EnviCli, vivisect.VivWorkspace):
     '''
     A class that builds upon the VivWorkspace to provide command line capabilities so that
     things like the Vivisect UI can provide a cleaner interface that just a direct python
@@ -79,10 +80,12 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
 
         cols, results = viv_reports.runReportModule(self, line)
         for va, row in results.items():
+            self.canvas.addVaText("%s:\n" % self.getName(va), va)
+
             for indx in range(len(cols)):
                 valu = row[indx]
                 name, typename = cols[indx]
-                self.canvas.addVaText(name, va)
+                self.canvas.addVaText("    " + name, va)
                 self.canvas.addText(": %s\n" % valu)
             self.canvas.addText("\n")
 
@@ -94,9 +97,16 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
 
         Usage: pathcount <func_expr>
         '''
-        fva = self.parseExpression(line)
-        if not self.isFunction(fva):
-            self.vprint('Not a function!')
+        if not line:
+            return self.do_help('pathcount')
+        try:
+            fva = self.parseExpression(line)
+            if not self.isFunction(fva):
+                self.vprint('Not a function!')
+                return
+
+        except Exception as e:
+            self.vprint(str(e))
             return
 
         g = v_t_graph.buildFunctionGraph(self, fva)
@@ -116,6 +126,8 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
             to the given address
 
         '''
+        if not line:
+            return self.do_help("symboliks")
 
         watchaddr = None
 
@@ -156,7 +168,7 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
                 if emu.isLocalMemory(addrsym):
                     continue
                 self.vprint('[ %s ] = %s' % (addrsym, valsym))
-            self.vprint('RETURN', emu.getFunctionReturn().reduce())
+            self.vprint('RETURN  %r' % emu.getFunctionReturn().reduce())
 
     def do_names(self, line):
         '''
@@ -265,7 +277,7 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
 
             for nva, node in graph.getNodes():
                 va = nva
-                endva = va + node.get('cbsize')
+                endva = nva + node.get('cbsize')
                 while va < endva:
                     lva, lsz, ltype, ltinfo = self.getLocation(va)
                     valist.append(va)
@@ -367,13 +379,16 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
             pname = e_memory.reprPerms(mperm)
             sname = self.reprPointer(va)
 
-            op = self.parseOpcode(va)
-            self.canvas.renderMemory(va, len(op))
-            cmt = self.getComment(va)
-            if cmt is not None:
-                self.canvas.addText('\t\t; %s (Perms: %s, Smartname: %s)' % (cmt, pname, sname))
+            try:
+                op = self.parseOpcode(va)
+                self.canvas.renderMemory(va, len(op))
+                cmt = self.getComment(va)
+                if cmt is not None:
+                    self.canvas.addText('\t\t; %s (Perms: %s, Smartname: %s)' % (cmt, pname, sname))
 
-            self.canvas.addText('\n')
+                self.canvas.addText('\n')
+            except envi.SegmentationViolation as e:
+                logger.debug("segv at 0x%x", va)
 
         self.vprint('done (%d results).' % len(res))
 
@@ -452,12 +467,9 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
 
         Usage: exports [fname]
         """
-        edict = {}
+        edict = collections.defaultdict(list)
         for va, etype, name, filename in self.getExports():
-            exps = edict.get(filename)
-            if exps is None:
-                edict[filename] = []
-            exps.append((name, va))
+            edict[filename].append((name, va))
 
         if line:
             x = edict.get(line)
@@ -566,7 +578,7 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
         """
         argv = e_cli.splitargs(line)
         try:
-            opts, args = getopt(argv, "csup:S:")
+            opts, args = getopt(argv, "csufn:p:S:")
         except Exception as e:
             logger.warning(str(e))
             return self.do_help("make")
@@ -616,19 +628,22 @@ class VivCli(vivisect.VivWorkspace, e_cli.EnviCli):
         if not line:
             return self.do_help("emulate")
 
-        emu = self.getEmulator()
         addr = self.parseExpression(line)
-        emu.setProgramCounter(addr)
+        emu = self.getEmulator(va=addr)
 
         trace = vt_envitools.TraceEmulator(emu)
 
         db = vdb.Vdb(trace=trace)
         db.cmdloop()
 
-    def do_argtrack(self, line):
+    def _do_argtrack(self, line):
+        #FIXME: this is currently broken and needs to be revamped to use 
+        # apiCall and VivTaints
         """
         Track input arguments to the given function by name or address.
 
+        (this is currently broken.  sorry!)
+
         Usage: argtrack <func_addr_expr> <arg_idx>
         """
         if not line:
