@@ -37,7 +37,6 @@ def loadExtensions(vw, vwgui):
             modpath = os.path.join(dirname, fname)
             # dig into first level of directories and exec the __init__.py
             if os.path.isdir(modpath):
-                modname = fname
                 modpath = os.path.join(modpath, '__init__.py')
                 if not os.path.exists(modpath):
                     continue
@@ -46,10 +45,6 @@ def loadExtensions(vw, vwgui):
             elif not modpath.endswith('.py'):
                 continue
 
-            else:
-                # it's a .py file
-                modname = fname[:-3]
-
             try:
                 # Build code objects from the module files
                 spec = importlib.util.spec_from_file_location(fname, modpath)
