@@ -41,6 +41,11 @@ class WorkspaceEmulator:
         but most children like the ArmWorkspaceEmulator and any others that live in vivisect/impemu/platarm/
 
         Current Keyword Arguments:
+        * va
+            - Type: Integer
+            - Default: None
+            - Desc: The address to begin emulating from.  emu.setProgramCounter() is called with this value,
+                    and in the case of ArmWorkspaceEmulator, appropriate architecture is set.
         * logwrite
             - Type: Boolean
             - Default: False
@@ -169,6 +174,11 @@ class WorkspaceEmulator:
         self.stack_pointer = None
         self.initStackMemory()
 
+        # set Program Counter if provided
+        va = kwargs.get('va')
+        if va:
+            self.setProgramCounter(va)
+
     def initStackMemory(self, stacksize=init_stack_size):
         '''
         Setup and initialize stack memory.
@@ -486,6 +496,7 @@ class WorkspaceEmulator:
 
                         if self.emustop:
                             return
+
                     iscall = self.checkCall(starteip, endeip, op)
                     if self.emustop:
                         return
@@ -654,6 +665,10 @@ class WorkspaceEmulator:
                 if val not in argv:
                     return self.reprVivTaint(taint)
 
+            else:
+                return self.reprVivTaint(taint)
+
+
         stackoff = self.getStackOffset(val)
         if stackoff is not None:
             funclocal = self.vw.getFunctionLocal(self.funcva, stackoff)
